// SystemMutex
//------------------------------------------------------------------------------

// Includes
//------------------------------------------------------------------------------
#include "Core/PrecompiledHeader.h"

#include "SystemMutex.h"

#if defined( __WINDOWS__ )
    #include <windows.h>
    #include "Core/Env/Assert.h"
#endif

// CONSTRUCTOR
//------------------------------------------------------------------------------
SystemMutex::SystemMutex( const char * name )
	: m_Handle( INVALID_HANDLE_VALUE )
	, m_Name( name )
{
}

// DESTRUCTOR
//------------------------------------------------------------------------------
SystemMutex::~SystemMutex()
{
	if ( m_Handle != INVALID_HANDLE_VALUE )
	{
		Unlock();
	}
}

// Lock
//------------------------------------------------------------------------------
bool SystemMutex::TryLock()
{ 
    #if defined( __WINDOWS__ )
		void * handle = (void *)CreateMutex( nullptr, TRUE, m_Name.Get() );
		if ( GetLastError() == ERROR_ALREADY_EXISTS )
		{
			if ( ( handle != INVALID_HANDLE_VALUE ) && ( handle != 0 ) )
			{
				CloseHandle( handle );
			}
			return false;
		}
		m_Handle = handle;
		return true;
    #elif defined( __APPLE__ )
        // TODO:MAC Implement SystemMutex
		return false;
    #else
        #error Unknown platform
    #endif
}
// Unlock
//------------------------------------------------------------------------------
void SystemMutex::Unlock()
{
    #if defined( __WINDOWS__ )
		ASSERT( m_Handle != INVALID_HANDLE_VALUE );
		CloseHandle( m_Handle );
		m_Handle = INVALID_HANDLE_VALUE;
    #elif defined( __APPLE__ )
        // TODO:MAC Implement SystemMutex
    #else
        #error Unknown platform
    #endif
}

//------------------------------------------------------------------------------
